package com.yeejoin.amos.fas.business.service.intfc;

import com.yeejoin.amos.fas.dao.entity.TopographyNode;
import com.yeejoin.amos.fas.dao.entity.TopographyNodeDetail;

import java.util.List;

public interface ITopographyNodeService {

	public List<TopographyNode> getNodesByAppIdAndType(String appId,String type);

	public void saveNodes(List<TopographyNode> nodeData);

	public TopographyNodeDetail getNodeDetailById(String nodeid);

	public TopographyNodeDetail saveNodeDetail(TopographyNodeDetail nodeDetail);

	public void deleteNode(TopographyNode node);

	public TopographyNode queryByKeyAndAppIdAndType(String nodekey, String appId,String type);

	public int queryMaxKeyByAppIdAndType(String appId,String type);

	public TopographyNode queryById(String id);

	public List<TopographyNode> getNodesByType(String type);

}
